@@ -6,12 +6,14 @@ use 5.014;
 
 no if $] >= 5.018, warnings => 'experimental::smartmatch';
 
-our $VERSION = '1.05';
+our $VERSION = '1.07';
 
 use Carp qw(confess cluck);
 use DateTime;
 use Encode qw(encode decode);
 use List::Util qw(first);
+use List::MoreUtils qw(uniq);
+use List::UtilsBy qw(uniq_by);
 use LWP::UserAgent;
 use Travel::Status::DE::IRIS::Result;
 use XML::LibXML;
@@ -157,66 +159,80 @@ sub get_with_cache {
 sub get_station {
 	my ( $self, %opt ) = @_;
 
+	my $iter_depth = 0;
 	my @ret;
-	my $recursion_depth = $opt{recursion_depth} // 0;
+	my @queue = ( $opt{name} );
+	my @seen;
 
-	my ( $raw, $err )
-	  = $self->get_with_cache( $self->{main_cache},
-		$self->{iris_base} . '/station/' . $opt{name} );
-	if ($err) {
-		$self->{errstr} = "Failed to fetch station data: $err";
-		return;
-	}
+	while ( @queue and $iter_depth < 12 ) {
+		my $station = shift(@queue);
+		push( @seen, $station );
+		$iter_depth++;
 
-	my $xml_st = XML::LibXML->load_xml( string => $raw );
-
-	my $station_node = ( $xml_st->findnodes('//station') )[0];
-
-	if ( not $station_node ) {
-		if ( $opt{root} ) {
-			$self->{errstr}
-			  = "The station '$opt{name}' has no associated timetable";
-		}
-		return;
-	}
-	if ( $recursion_depth > 5 ) {
-		cluck("Reached recursion depth $recursion_depth while tracking IDs");
-		return;
-	}
-
-	push(
-		@ret,
-		{
-			uic   => $station_node->getAttribute('eva'),
-			name  => $station_node->getAttribute('name'),
-			ds100 => $station_node->getAttribute('ds100'),
-		}
-	);
-
-	if ( $self->{developer_mode} ) {
-		printf( " -> %s (%s / %s)\n", @{ $ret[0] }{qw{name uic ds100}} );
-	}
-
-	if ( $opt{recursive} and $station_node->hasAttribute('meta') ) {
-		my @recursion_blacklist = @{ $opt{recursion_blacklist} // [] };
-		my @refs = split( m{ \| }x, $station_node->getAttribute('meta') );
-
-		push( @recursion_blacklist, $ret[0]{uic} );
-		for my $ref (@refs) {
-			if ( not( $ref ~~ \@recursion_blacklist ) ) {
-				push(
-					@ret,
-					$self->get_station(
-						name                => $ref,
-						recursive           => 1,
-						recursion_depth     => $recursion_depth + 1,
-						recursion_blacklist => \@recursion_blacklist,
-					)
-				);
+		my ( $raw, $err )
+		  = $self->get_with_cache( $self->{main_cache},
+			$self->{iris_base} . '/station/' . $station );
+		if ($err) {
+			if ( $opt{root} ) {
+				$self->{errstr} = "Failed to fetch station data: $err";
+				return;
+			}
+			else {
+				$self->{warnstr}
+				  = "Failed to fetch station data for '$station': $err\n";
+				next;
 			}
 		}
+
+		my $xml_st = XML::LibXML->load_xml( string => $raw );
+
+		my $station_node = ( $xml_st->findnodes('//station') )[0];
+
+		if ( not $station_node ) {
+			if ( $opt{root} ) {
+				$self->{errstr}
+				  = "Station '$station' has no associated timetable";
+				return;
+			}
+			else {
+				$self->{warnstr}
+				  = "Station '$station' has no associated timetable";
+				next;
+			}
+			next;
+		}
+
+		push( @seen, $station_node->getAttribute('eva') );
+
+		push(
+			@ret,
+			{
+				uic   => $station_node->getAttribute('eva'),
+				name  => $station_node->getAttribute('name'),
+				ds100 => $station_node->getAttribute('ds100'),
+			}
+		);
+
+		if ( $self->{developer_mode} ) {
+			printf( " -> %s (%s / %s)\n", @{ $ret[-1] }{qw{name uic ds100}} );
+		}
+
+		if ( $opt{recursive} ) {
+			my @refs
+			  = uniq( split( m{ \| }x, $station_node->getAttribute('meta') ) );
+			@refs = grep { not( $_ ~~ \@seen or $_ ~~ \@queue ) } @refs;
+			push( @queue, @refs );
+			$opt{root} = 0;
+		}
 	}
 
+	if (@queue) {
+		cluck(  "Reached $iter_depth iterations when tracking station IDs. "
+			  . "This is probably a bug" );
+	}
+
+	@ret = uniq_by { $_->{uic} } @ret;
+
 	return @ret;
 }
 
@@ -520,7 +536,7 @@ Travel::Status::DE::IRIS - Interface to IRIS based web departure monitors.
 
 =head1 VERSION
 
-version 1.05
+version 1.07
 
 =head1 DESCRIPTION
 
@@ -574,6 +590,17 @@ station name, but a station code, such as "EE" (for Essen Hbf) or "KA"
 (for Aachen Hbf). See Travel::Status::DE::IRIS::Stations(3pm) for a
 name to code mapping.
 
+=item B<with_related> => I<bool>
+
+Sometimes, Deutsche Bahn splits up major stations in the IRIS interface.  For
+instance, "KE<ouml>ln Messe/Deutz" actually consists of "KE<ouml>ln
+Messe/Deutz" (KKDZ), "KE<ouml>ln Messe/Deutz Gl. 9-10" (KKDZB) and "KE<ouml>ln
+Messe/Deutz (tief)" (KKDT).
+
+By default, Travel::Status::DE::IRIS only returns departures for the specified
+station. When this option is set to a true value, it will also return
+departures for all related stations.
+
 =back
 
 =item $status->errstr
